! ***********************************************************************
!
!   Copyright (C) 2010  Bill Paxton
!
!   this file is part of mesa.
!
!   mesa is free software; you can redistribute it and/or modify
!   it under the terms of the gnu general library public license as published
!   by the free software foundation; either version 2 of the license, or
!   (at your option) any later version.
!
!   mesa is distributed in the hope that it will be useful, 
!   but without any warranty; without even the implied warranty of
!   merchantability or fitness for a particular purpose.  see the
!   gnu library general public license for more details.
!
!   you should have received a copy of the gnu library general public license
!   along with this software; if not, write to the free software
!   foundation, inc., 59 temple place, suite 330, boston, ma 02111-1307 usa
!
! ***********************************************************************
 
      module run_star_extras

      use star_lib
      use star_def
      use const_def
      use math_lib
      use gyre_lib
      use utils_lib

      implicit none

      include "test_suite_extras_def.inc"

    real(dp), target, dimension(:) :: actual_Qs(1296) !=>NULL()
    real(dp), target, dimension(:) :: actual_neuQs(1296) !=>NULL() ! ignore if null  (num_reactions)
    logical, target :: from_weaklib(1296)! ! ignore if null
!
!      real(dp) :: temp49
!      real(dp) :: temp50
!      real(dp) :: temp51
!      real(dp) :: temp52
!      real(dp) :: temp53
!      real(dp) :: temp54
!      real(dp) :: temp55
!      real(dp) :: temp56
!      real(dp) :: temp57

! here are the x controls used below

!gyre
!      x_logical_ctrl(37) = .false. ! if true, then run GYRE
!      x_integer_ctrl(1) = 2 ! output GYRE info at this step interval
!      x_logical_ctrl(1) = .false. ! save GYRE info whenever save profile
!      x_integer_ctrl(2) = 2 ! max number of modes to output per call
!      x_logical_ctrl(2) = .false. ! output eigenfunction files
!      x_integer_ctrl(3) = 0 ! mode l (e.g. 0 for p modes, 1 for g modes)
!      x_integer_ctrl(4) = 1 ! order
!      x_ctrl(1) = 0.158d-05 ! freq ~ this (Hz)
!      x_ctrl(2) = 0.33d+03 ! growth < this (days)

!alpha_mlt_routine
         !alpha_H = s% x_ctrl(21)
         !alpha_other = s% x_ctrl(22)
         !H_limit = s% x_ctrl(23)
      
      contains

      include "test_suite_extras.inc"
      
      
      subroutine extras_controls(id, ierr)
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         s% extras_startup => extras_startup
         s% extras_check_model => extras_check_model
         s% extras_finish_step => extras_finish_step
         s% extras_after_evolve => extras_after_evolve
         s% how_many_extra_history_columns => how_many_extra_history_columns
         s% data_for_extra_history_columns => data_for_extra_history_columns
         s% how_many_extra_profile_columns => how_many_extra_profile_columns
         s% data_for_extra_profile_columns => data_for_extra_profile_columns  
         s% other_alpha_mlt => alpha_mlt_routine       
      end subroutine extras_controls


      subroutine alpha_mlt_routine(id, ierr)
         use chem_def, only: ih1
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         integer :: k, h1
         real(dp) :: alpha_H, alpha_other, H_limit
         include 'formats'
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         alpha_H = s% x_ctrl(21)
         alpha_other = s% x_ctrl(22)
         H_limit = s% x_ctrl(23)
         h1 = s% net_iso(ih1)
         !write(*,1) 'alpha_H', alpha_H
         !write(*,1) 'alpha_other', alpha_other
         !write(*,1) 'H_limit', H_limit
         !write(*,2) 'h1', h1
         !write(*,2) 's% nz', s% nz
         if (alpha_H <= 0 .or. alpha_other <= 0 .or. h1 <= 0) return
         do k=1,s% nz
            if (s% xa(h1,k) >= H_limit) then
               s% alpha_mlt(k) = alpha_H
            else
               s% alpha_mlt(k) = alpha_other
            end if
            !write(*,2) 'alpha_mlt', k, s% alpha_mlt(k), 
         end do
         !stop
      end subroutine alpha_mlt_routine
      
      
      subroutine extras_startup(id, restart, ierr)
         integer, intent(in) :: id
         logical, intent(in) :: restart
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         call test_suite_startup(s, restart, ierr)         
            
         if (.not. s% x_logical_ctrl(37)) return
         
         ! Initialize GYRE

         call gyre_init('gyre.in')

         ! Set constants

         call gyre_set_constant('G_GRAVITY', standard_cgrav)
         call gyre_set_constant('C_LIGHT', clight)
         call gyre_set_constant('A_RADIATION', crad)

         call gyre_set_constant('M_SUN', Msun)
         call gyre_set_constant('R_SUN', Rsun)
         call gyre_set_constant('L_SUN', Lsun)

         call gyre_set_constant('GYRE_DIR', TRIM(mesa_dir)//'/gyre/gyre')

      end subroutine extras_startup
      
      
      subroutine extras_after_evolve(id, ierr)
         use num_lib, only: find0
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         real(dp) :: dt, m
         integer :: k, nz
         include 'formats'
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         nz = s% nz
         write(*,*)
         call test_suite_after_evolve(s, ierr)
         if (ierr /= 0) return         
         if (.not. s% x_logical_ctrl(37)) return
         call gyre_final()
      end subroutine extras_after_evolve
      

      ! returns either keep_going, retry, or terminate.
      integer function extras_check_model(id)
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_check_model = keep_going         
      end function extras_check_model


      integer function how_many_extra_history_columns(id)
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_history_columns = 306
      end function how_many_extra_history_columns


subroutine data_for_extra_history_columns(id, n, names, vals, ierr)

   use star_def, only: star_info
   use const_def, only: dp
   use net_lib, only: net_work_size
   use rates_lib, only: rates_reaction_id
   integer, intent(in) :: id, n
   character (len=maxlen_history_column_name) :: names(n)
   real(dp) :: vals(n), temp(n), frac(n),baryonic_mass(n)
   real(dp) :: integral_norm, gamma1_integral, P_tot
   real(dp) :: nps_beta, nps_invbeta, eps_beta, eps_invbeta
   integer, intent(out) :: ierr
   integer :: num_weak_reactions
   integer :: i, k, op_err, net_lwork
   logical :: okay
   character (len=maxlen_history_column_name) :: names_weak_reactions(n)
   character (len=maxlen_history_column_name) :: names_weak_ps(n)
   type (star_info), pointer :: s
   ierr = 0
   call star_ptr(id, s, ierr)
   if (ierr /= 0) return

num_weak_reactions = 148


! new variables will be placed above the reactions.
! in low mass inlist, num_weak_reactions =

! 78 beta decay reactions
names_weak_reactions(1) = 'r_h1_he3_wk_he4'
names_weak_reactions(2) = 'r_h1_h1_ec_h2'
names_weak_reactions(3) = 'r_h1_h1_wk_h2'
names_weak_reactions(4) = 'r_h1_wk_neut'

names_weak_reactions(5) = 'r_be7_wk_li7'

names_weak_reactions(6) = 'r_b8_wk_he4_he4'

names_weak_reactions(7) = 'r_n13_wk_c13'

names_weak_reactions(8) = 'r_o14_wk_n14'
names_weak_reactions(9) = 'r_o15_wk_n15'

names_weak_reactions(10) = 'r_f17_wk_o17'
names_weak_reactions(11) = 'r_f18_wk_o18'

names_weak_reactions(12) = 'r_ne18_wk_f18'
names_weak_reactions(13) = 'r_ne19_wk_f19'
names_weak_reactions(14) = 'r_ne20_wk_f20'
names_weak_reactions(15) = 'r_ne23_wk_f23'

names_weak_reactions(16) = 'r_na21_wk_ne21'
names_weak_reactions(17) = 'r_na22_wk_ne22'
names_weak_reactions(18) = 'r_na23_wk_ne23'
names_weak_reactions(19) = 'r_na24_wk_ne24'

names_weak_reactions(20) = 'r_mg22_wk_na22'
names_weak_reactions(21) = 'r_mg23_wk_na23'
names_weak_reactions(22) = 'r_mg24_wk_na24'
names_weak_reactions(23) = 'r_mg25_wk_na25'

names_weak_reactions(24) = 'r_al25_wk_mg25'
names_weak_reactions(25) = 'r_al26_wk_mg26'
names_weak_reactions(26) = 'r_al27_wk_mg27'

names_weak_reactions(27) = 'r_si26_wk_al26'
names_weak_reactions(28) = 'r_si27_wk_al27'
names_weak_reactions(29) = 'r_si28_wk_al28'

names_weak_reactions(30) = 'r_p30_wk_si30'

names_weak_reactions(31) = 'r_s31_wk_p31'
names_weak_reactions(32) = 'r_s32_wk_p32'

names_weak_reactions(33) = 'r_ar35_wk_cl35'
names_weak_reactions(34) = 'r_ar36_wk_cl36'
names_weak_reactions(35) = 'r_ar37_wk_cl37'

names_weak_reactions(36) = 'r_ca39_wk_k39'
names_weak_reactions(37) = 'r_ca40_wk_k40'
names_weak_reactions(38) = 'r_ca41_wk_k41'
names_weak_reactions(39) = 'r_ca42_wk_k42'
names_weak_reactions(40) = 'r_ca43_wk_k43'

names_weak_reactions(41) = 'r_sc43_wk_ca43'
names_weak_reactions(42) = 'r_sc44_wk_ca44'

names_weak_reactions(43) = 'r_ti44_wk_sc44'
names_weak_reactions(44) = 'r_ti45_wk_sc45'
names_weak_reactions(45) = 'r_ti46_wk_sc46'

names_weak_reactions(46) = 'r_v47_wk_ti47'
names_weak_reactions(47) = 'r_v48_wk_ti48'

names_weak_reactions(48) = 'r_cr48_wk_v48'
names_weak_reactions(49) = 'r_cr49_wk_v49'
names_weak_reactions(50) = 'r_cr50_wk_v50'
names_weak_reactions(51) = 'r_cr51_wk_v51'

names_weak_reactions(52) = 'r_mn51_wk_cr51'
names_weak_reactions(53) = 'r_mn52_wk_cr52'
names_weak_reactions(54) = 'r_mn53_wk_cr53'
names_weak_reactions(55) = 'r_mn54_wk_cr54'
names_weak_reactions(56) = 'r_mn55_wk_cr55'
names_weak_reactions(57) = 'r_mn56_wk_cr56'

names_weak_reactions(58) = 'r_fe52_wk_mn52'
names_weak_reactions(59) = 'r_fe53_wk_mn53'
names_weak_reactions(60) = 'r_fe54_wk_mn54'
names_weak_reactions(61) = 'r_fe55_wk_mn55'
names_weak_reactions(62) = 'r_fe56_wk_mn56'

names_weak_reactions(63) = 'r_co55_wk_fe55'
names_weak_reactions(64) = 'r_co56_wk_fe56'
names_weak_reactions(65) = 'r_co57_wk_fe57'
names_weak_reactions(66) = 'r_co58_wk_fe58'

names_weak_reactions(67) = 'r_ni55_wk_co55'
names_weak_reactions(68) = 'r_ni56_wk_co56'
names_weak_reactions(69) = 'r_ni57_wk_co57'
names_weak_reactions(70) = 'r_ni58_wk_co58'
names_weak_reactions(71) = 'r_ni59_wk_co59'
names_weak_reactions(72) = 'r_ni60_wk_co60'

names_weak_reactions(73) = 'r_cu59_wk_ni59'
names_weak_reactions(74) = 'r_cu60_wk_ni60'
names_weak_reactions(75) = 'r_cu61_wk_ni61'

names_weak_reactions(76) = 'r_zn60_wk_cu60'
names_weak_reactions(77) = 'r_zn61_wk_cu61'
names_weak_reactions(78) = 'r_zn62_wk_cu62'

! 70 inverse beta decay reactions
names_weak_reactions(79) = 'r_neut_wk-minus_h1'

names_weak_reactions(80) = 'r_o17_wk-minus_f17'
names_weak_reactions(81) = 'r_o18_wk-minus_f18'

names_weak_reactions(82) = 'r_f18_wk-minus_ne18'
names_weak_reactions(83) = 'r_f19_wk-minus_ne19'
names_weak_reactions(84) = 'r_f20_wk-minus_ne20'
names_weak_reactions(85) = 'r_f23_wk-minus_ne23'

names_weak_reactions(86) = 'r_ne21_wk-minus_na21'
names_weak_reactions(87) = 'r_ne22_wk-minus_na22'
names_weak_reactions(88) = 'r_ne23_wk-minus_na23'
names_weak_reactions(89) = 'r_ne24_wk-minus_na24'

names_weak_reactions(90) = 'r_na22_wk-minus_mg22'
names_weak_reactions(91) = 'r_na23_wk-minus_mg23'
names_weak_reactions(92) = 'r_na24_wk-minus_mg24'
names_weak_reactions(93) = 'r_na25_wk-minus_mg25'

names_weak_reactions(94) = 'r_mg25_wk-minus_al25'
names_weak_reactions(95) = 'r_mg26_wk-minus_al26'
names_weak_reactions(96) = 'r_mg27_wk-minus_al27'

names_weak_reactions(97) = 'r_al26_wk-minus_si26'
names_weak_reactions(98) = 'r_al27_wk-minus_si27'
names_weak_reactions(99) = 'r_al28_wk-minus_si28'

names_weak_reactions(100) = 'r_si30_wk-minus_p30'

names_weak_reactions(101) = 'r_p31_wk-minus_s31'
names_weak_reactions(102) = 'r_p32_wk-minus_s32'

names_weak_reactions(103) = 'r_cl35_wk-minus_ar35'
names_weak_reactions(104) = 'r_cl36_wk-minus_ar36'
names_weak_reactions(105) = 'r_cl37_wk-minus_ar37'

names_weak_reactions(106) = 'r_k39_wk-minus_ca39'
names_weak_reactions(107) = 'r_k40_wk-minus_ca40'
names_weak_reactions(108) = 'r_k41_wk-minus_ca41'
names_weak_reactions(109) = 'r_k42_wk-minus_ca42'
names_weak_reactions(110) = 'r_k43_wk-minus_ca43'

names_weak_reactions(111) = 'r_ca43_wk-minus_sc43'
names_weak_reactions(112) = 'r_ca44_wk-minus_sc44'

names_weak_reactions(113) = 'r_sc44_wk-minus_ti44'
names_weak_reactions(114) = 'r_sc45_wk-minus_ti45'
names_weak_reactions(115) = 'r_sc46_wk-minus_ti46'

names_weak_reactions(116) = 'r_ti47_wk-minus_v47'
names_weak_reactions(117) = 'r_ti48_wk-minus_v48'

names_weak_reactions(118) = 'r_v48_wk-minus_cr48'
names_weak_reactions(119) = 'r_v49_wk-minus_cr49'
names_weak_reactions(120) = 'r_v50_wk-minus_cr50'
names_weak_reactions(121) = 'r_v51_wk-minus_cr51'

names_weak_reactions(122) = 'r_cr51_wk-minus_mn51'
names_weak_reactions(123) = 'r_cr52_wk-minus_mn52'
names_weak_reactions(124) = 'r_cr53_wk-minus_mn53'
names_weak_reactions(125) = 'r_cr54_wk-minus_mn54'
names_weak_reactions(126) = 'r_cr55_wk-minus_mn55'
names_weak_reactions(127) = 'r_cr56_wk-minus_mn56'

names_weak_reactions(128) = 'r_mn52_wk-minus_fe52'
names_weak_reactions(129) = 'r_mn53_wk-minus_fe53'
names_weak_reactions(130) = 'r_mn54_wk-minus_fe54'
names_weak_reactions(131) = 'r_mn55_wk-minus_fe55'
names_weak_reactions(132) = 'r_mn56_wk-minus_fe56'

names_weak_reactions(133) = 'r_fe55_wk-minus_co55'
names_weak_reactions(134) = 'r_fe56_wk-minus_co56'
names_weak_reactions(135) = 'r_fe57_wk-minus_co57'
names_weak_reactions(136) = 'r_fe58_wk-minus_co58'

names_weak_reactions(137) = 'r_co55_wk-minus_ni55'
names_weak_reactions(138) = 'r_co56_wk-minus_ni56'
names_weak_reactions(139) = 'r_co57_wk-minus_ni57'
names_weak_reactions(140) = 'r_co58_wk-minus_ni58'
names_weak_reactions(141) = 'r_co59_wk-minus_ni59'
names_weak_reactions(142) = 'r_co60_wk-minus_ni60'

names_weak_reactions(143) = 'r_ni59_wk-minus_cu59'
names_weak_reactions(144) = 'r_ni60_wk-minus_cu60'
names_weak_reactions(145) = 'r_ni61_wk-minus_cu61'

names_weak_reactions(146) = 'r_cu60_wk-minus_zn60'
names_weak_reactions(147) = 'r_cu61_wk-minus_zn61'
names_weak_reactions(148) = 'r_cu62_wk-minus_zn62'


!neutrinos per second
! 78 beta decay reactions
names_weak_ps(1) = 'nps_h1_he3_wk_he4'
names_weak_ps(2) = 'nps_h1_h1_ec_h2'
names_weak_ps(3) = 'nps_h1_h1_wk_h2'
names_weak_ps(4) = 'nps_h1_wk_neut'

names_weak_ps(5) = 'nps_be7_wk_li7'

names_weak_ps(6) = 'nps_b8_wk_he4_he4'

names_weak_ps(7) = 'nps_n13_wk_c13'

names_weak_ps(8) = 'nps_o14_wk_n14'
names_weak_ps(9) = 'nps_o15_wk_n15'

names_weak_ps(10) = 'nps_f17_wk_o17'
names_weak_ps(11) = 'nps_f18_wk_o18'

names_weak_ps(12) = 'nps_ne18_wk_f18'
names_weak_ps(13) = 'nps_ne19_wk_f19'
names_weak_ps(14) = 'nps_ne20_wk_f20'
names_weak_ps(15) = 'nps_ne23_wk_f23'

names_weak_ps(16) = 'nps_na21_wk_ne21'
names_weak_ps(17) = 'nps_na22_wk_ne22'
names_weak_ps(18) = 'nps_na23_wk_ne23'
names_weak_ps(19) = 'nps_na24_wk_ne24'

names_weak_ps(20) = 'nps_mg22_wk_na22'
names_weak_ps(21) = 'nps_mg23_wk_na23'
names_weak_ps(22) = 'nps_mg24_wk_na24'
names_weak_ps(23) = 'nps_mg25_wk_na25'

names_weak_ps(24) = 'nps_al25_wk_mg25'
names_weak_ps(25) = 'nps_al26_wk_mg26'
names_weak_ps(26) = 'nps_al27_wk_mg27'

names_weak_ps(27) = 'nps_si26_wk_al26'
names_weak_ps(28) = 'nps_si27_wk_al27'
names_weak_ps(29) = 'nps_si28_wk_al28'

names_weak_ps(30) = 'nps_p30_wk_si30'

names_weak_ps(31) = 'nps_s31_wk_p31'
names_weak_ps(32) = 'nps_s32_wk_p32'

names_weak_ps(33) = 'nps_ar35_wk_cl35'
names_weak_ps(34) = 'nps_ar36_wk_cl36'
names_weak_ps(35) = 'nps_ar37_wk_cl37'

names_weak_ps(36) = 'nps_ca39_wk_k39'
names_weak_ps(37) = 'nps_ca40_wk_k40'
names_weak_ps(38) = 'nps_ca41_wk_k41'
names_weak_ps(39) = 'nps_ca42_wk_k42'
names_weak_ps(40) = 'nps_ca43_wk_k43'

names_weak_ps(41) = 'nps_sc43_wk_ca43'
names_weak_ps(42) = 'nps_sc44_wk_ca44'

names_weak_ps(43) = 'nps_ti44_wk_sc44'
names_weak_ps(44) = 'nps_ti45_wk_sc45'
names_weak_ps(45) = 'nps_ti46_wk_sc46'

names_weak_ps(46) = 'nps_v47_wk_ti47'
names_weak_ps(47) = 'nps_v48_wk_ti48'

names_weak_ps(48) = 'nps_cr48_wk_v48'
names_weak_ps(49) = 'nps_cr49_wk_v49'
names_weak_ps(50) = 'nps_cr50_wk_v50'
names_weak_ps(51) = 'nps_cr51_wk_v51'

names_weak_ps(52) = 'nps_mn51_wk_cr51'
names_weak_ps(53) = 'nps_mn52_wk_cr52'
names_weak_ps(54) = 'nps_mn53_wk_cr53'
names_weak_ps(55) = 'nps_mn54_wk_cr54'
names_weak_ps(56) = 'nps_mn55_wk_cr55'
names_weak_ps(57) = 'nps_mn56_wk_cr56'

names_weak_ps(58) = 'nps_fe52_wk_mn52'
names_weak_ps(59) = 'nps_fe53_wk_mn53'
names_weak_ps(60) = 'nps_fe54_wk_mn54'
names_weak_ps(61) = 'nps_fe55_wk_mn55'
names_weak_ps(62) = 'nps_fe56_wk_mn56'

names_weak_ps(63) = 'nps_co55_wk_fe55'
names_weak_ps(64) = 'nps_co56_wk_fe56'
names_weak_ps(65) = 'nps_co57_wk_fe57'
names_weak_ps(66) = 'nps_co58_wk_fe58'

names_weak_ps(67) = 'nps_ni55_wk_co55'
names_weak_ps(68) = 'nps_ni56_wk_co56'
names_weak_ps(69) = 'nps_ni57_wk_co57'
names_weak_ps(70) = 'nps_ni58_wk_co58'
names_weak_ps(71) = 'nps_ni59_wk_co59'
names_weak_ps(72) = 'nps_ni60_wk_co60'

names_weak_ps(73) = 'nps_cu59_wk_ni59'
names_weak_ps(74) = 'nps_cu60_wk_ni60'
names_weak_ps(75) = 'nps_cu61_wk_ni61'

names_weak_ps(76) = 'nps_zn60_wk_cu60'
names_weak_ps(77) = 'nps_zn61_wk_cu61'
names_weak_ps(78) = 'nps_zn62_wk_cu62'

! 70 inverse beta decay reactions
names_weak_ps(79) = 'nps_neut_wk-minus_h1'

names_weak_ps(80) = 'nps_o17_wk-minus_f17'
names_weak_ps(81) = 'nps_o18_wk-minus_f18'

names_weak_ps(82) = 'nps_f18_wk-minus_ne18'
names_weak_ps(83) = 'nps_f19_wk-minus_ne19'
names_weak_ps(84) = 'nps_f20_wk-minus_ne20'
names_weak_ps(85) = 'nps_f23_wk-minus_ne23'

names_weak_ps(86) = 'nps_ne21_wk-minus_na21'
names_weak_ps(87) = 'nps_ne22_wk-minus_na22'
names_weak_ps(88) = 'nps_ne23_wk-minus_na23'
names_weak_ps(89) = 'nps_ne24_wk-minus_na24'

names_weak_ps(90) = 'nps_na22_wk-minus_mg22'
names_weak_ps(91) = 'nps_na23_wk-minus_mg23'
names_weak_ps(92) = 'nps_na24_wk-minus_mg24'
names_weak_ps(93) = 'nps_na25_wk-minus_mg25'

names_weak_ps(94) = 'nps_mg25_wk-minus_al25'
names_weak_ps(95) = 'nps_mg26_wk-minus_al26'
names_weak_ps(96) = 'nps_mg27_wk-minus_al27'

names_weak_ps(97) = 'nps_al26_wk-minus_si26'
names_weak_ps(98) = 'nps_al27_wk-minus_si27'
names_weak_ps(99) = 'nps_al28_wk-minus_si28'

names_weak_ps(100) = 'nps_si30_wk-minus_p30'

names_weak_ps(101) = 'nps_p31_wk-minus_s31'
names_weak_ps(102) = 'nps_p32_wk-minus_s32'

names_weak_ps(103) = 'nps_cl35_wk-minus_ar35'
names_weak_ps(104) = 'nps_cl36_wk-minus_ar36'
names_weak_ps(105) = 'nps_cl37_wk-minus_ar37'

names_weak_ps(106) = 'nps_k39_wk-minus_ca39'
names_weak_ps(107) = 'nps_k40_wk-minus_ca40'
names_weak_ps(108) = 'nps_k41_wk-minus_ca41'
names_weak_ps(109) = 'nps_k42_wk-minus_ca42'
names_weak_ps(110) = 'nps_k43_wk-minus_ca43'

names_weak_ps(111) = 'nps_ca43_wk-minus_sc43'
names_weak_ps(112) = 'nps_ca44_wk-minus_sc44'

names_weak_ps(113) = 'nps_sc44_wk-minus_ti44'
names_weak_ps(114) = 'nps_sc45_wk-minus_ti45'
names_weak_ps(115) = 'nps_sc46_wk-minus_ti46'

names_weak_ps(116) = 'nps_ti47_wk-minus_v47'
names_weak_ps(117) = 'nps_ti48_wk-minus_v48'

names_weak_ps(118) = 'nps_v48_wk-minus_cr48'
names_weak_ps(119) = 'nps_v49_wk-minus_cr49'
names_weak_ps(120) = 'nps_v50_wk-minus_cr50'
names_weak_ps(121) = 'nps_v51_wk-minus_cr51'

names_weak_ps(122) = 'nps_cr51_wk-minus_mn51'
names_weak_ps(123) = 'nps_cr52_wk-minus_mn52'
names_weak_ps(124) = 'nps_cr53_wk-minus_mn53'
names_weak_ps(125) = 'nps_cr54_wk-minus_mn54'
names_weak_ps(126) = 'nps_cr55_wk-minus_mn55'
names_weak_ps(127) = 'nps_cr56_wk-minus_mn56'

names_weak_ps(128) = 'nps_mn52_wk-minus_fe52'
names_weak_ps(129) = 'nps_mn53_wk-minus_fe53'
names_weak_ps(130) = 'nps_mn54_wk-minus_fe54'
names_weak_ps(131) = 'nps_mn55_wk-minus_fe55'
names_weak_ps(132) = 'nps_mn56_wk-minus_fe56'

names_weak_ps(133) = 'nps_fe55_wk-minus_co55'
names_weak_ps(134) = 'nps_fe56_wk-minus_co56'
names_weak_ps(135) = 'nps_fe57_wk-minus_co57'
names_weak_ps(136) = 'nps_fe58_wk-minus_co58'

names_weak_ps(137) = 'nps_co55_wk-minus_ni55'
names_weak_ps(138) = 'nps_co56_wk-minus_ni56'
names_weak_ps(139) = 'nps_co57_wk-minus_ni57'
names_weak_ps(140) = 'nps_co58_wk-minus_ni58'
names_weak_ps(141) = 'nps_co59_wk-minus_ni59'
names_weak_ps(142) = 'nps_co60_wk-minus_ni60'

names_weak_ps(143) = 'nps_ni59_wk-minus_cu59'
names_weak_ps(144) = 'nps_ni60_wk-minus_cu60'
names_weak_ps(145) = 'nps_ni61_wk-minus_cu61'

names_weak_ps(146) = 'nps_cu60_wk-minus_zn60'
names_weak_ps(147) = 'nps_cu61_wk-minus_zn61'
names_weak_ps(148) = 'nps_cu62_wk-minus_zn62'



!erg per second from each weak reactions
! 78 beta decay reactions
names_weak_ps(1+ num_weak_reactions) = 'eps_h1_he3_wk_he4'
names_weak_ps(2+ num_weak_reactions) = 'eps_h1_h1_ec_h2'
names_weak_ps(3+ num_weak_reactions) = 'eps_h1_h1_wk_h2'
names_weak_ps(4+ num_weak_reactions) = 'eps_h1_wk_neut'

names_weak_ps(5+ num_weak_reactions) = 'eps_be7_wk_li7'

names_weak_ps(6+ num_weak_reactions) = 'eps_b8_wk_he4_he4'

names_weak_ps(7+ num_weak_reactions) = 'eps_n13_wk_c13'

names_weak_ps(8+ num_weak_reactions) = 'eps_o14_wk_n14'
names_weak_ps(9+ num_weak_reactions) = 'eps_o15_wk_n15'

names_weak_ps(10+ num_weak_reactions) = 'eps_f17_wk_o17'
names_weak_ps(11+ num_weak_reactions) = 'eps_f18_wk_o18'

names_weak_ps(12+ num_weak_reactions) = 'eps_ne18_wk_f18'
names_weak_ps(13+ num_weak_reactions) = 'eps_ne19_wk_f19'
names_weak_ps(14+ num_weak_reactions) = 'eps_ne20_wk_f20'
names_weak_ps(15+ num_weak_reactions) = 'eps_ne23_wk_f23'

names_weak_ps(16+ num_weak_reactions) = 'eps_na21_wk_ne21'
names_weak_ps(17+ num_weak_reactions) = 'eps_na22_wk_ne22'
names_weak_ps(18+ num_weak_reactions) = 'eps_na23_wk_ne23'
names_weak_ps(19+ num_weak_reactions) = 'eps_na24_wk_ne24'

names_weak_ps(20+ num_weak_reactions) = 'eps_mg22_wk_na22'
names_weak_ps(21+ num_weak_reactions) = 'eps_mg23_wk_na23'
names_weak_ps(22+ num_weak_reactions) = 'eps_mg24_wk_na24'
names_weak_ps(23+ num_weak_reactions) = 'eps_mg25_wk_na25'

names_weak_ps(24+ num_weak_reactions) = 'eps_al25_wk_mg25'
names_weak_ps(25+ num_weak_reactions) = 'eps_al26_wk_mg26'
names_weak_ps(26+ num_weak_reactions) = 'eps_al27_wk_mg27'

names_weak_ps(27+ num_weak_reactions) = 'eps_si26_wk_al26'
names_weak_ps(28+ num_weak_reactions) = 'eps_si27_wk_al27'
names_weak_ps(29+ num_weak_reactions) = 'eps_si28_wk_al28'

names_weak_ps(30+ num_weak_reactions) = 'eps_p30_wk_si30'

names_weak_ps(31+ num_weak_reactions) = 'eps_s31_wk_p31'
names_weak_ps(32+ num_weak_reactions) = 'eps_s32_wk_p32'

names_weak_ps(33+ num_weak_reactions) = 'eps_ar35_wk_cl35'
names_weak_ps(34+ num_weak_reactions) = 'eps_ar36_wk_cl36'
names_weak_ps(35+ num_weak_reactions) = 'eps_ar37_wk_cl37'

names_weak_ps(36+ num_weak_reactions) = 'eps_ca39_wk_k39'
names_weak_ps(37+ num_weak_reactions) = 'eps_ca40_wk_k40'
names_weak_ps(38+ num_weak_reactions) = 'eps_ca41_wk_k41'
names_weak_ps(39+ num_weak_reactions) = 'eps_ca42_wk_k42'
names_weak_ps(40+ num_weak_reactions) = 'eps_ca43_wk_k43'

names_weak_ps(41+ num_weak_reactions) = 'eps_sc43_wk_ca43'
names_weak_ps(42+ num_weak_reactions) = 'eps_sc44_wk_ca44'

names_weak_ps(43+ num_weak_reactions) = 'eps_ti44_wk_sc44'
names_weak_ps(44+ num_weak_reactions) = 'eps_ti45_wk_sc45'
names_weak_ps(45+ num_weak_reactions) = 'eps_ti46_wk_sc46'

names_weak_ps(46+ num_weak_reactions) = 'eps_v47_wk_ti47'
names_weak_ps(47+ num_weak_reactions) = 'eps_v48_wk_ti48'

names_weak_ps(48+ num_weak_reactions) = 'eps_cr48_wk_v48'
names_weak_ps(49+ num_weak_reactions) = 'eps_cr49_wk_v49'
names_weak_ps(50+ num_weak_reactions) = 'eps_cr50_wk_v50'
names_weak_ps(51+ num_weak_reactions) = 'eps_cr51_wk_v51'

names_weak_ps(52+ num_weak_reactions) = 'eps_mn51_wk_cr51'
names_weak_ps(53+ num_weak_reactions) = 'eps_mn52_wk_cr52'
names_weak_ps(54+ num_weak_reactions) = 'eps_mn53_wk_cr53'
names_weak_ps(55+ num_weak_reactions) = 'eps_mn54_wk_cr54'
names_weak_ps(56+ num_weak_reactions) = 'eps_mn55_wk_cr55'
names_weak_ps(57+ num_weak_reactions) = 'eps_mn56_wk_cr56'

names_weak_ps(58+ num_weak_reactions) = 'eps_fe52_wk_mn52'
names_weak_ps(59+ num_weak_reactions) = 'eps_fe53_wk_mn53'
names_weak_ps(60+ num_weak_reactions) = 'eps_fe54_wk_mn54'
names_weak_ps(61+ num_weak_reactions) = 'eps_fe55_wk_mn55'
names_weak_ps(62+ num_weak_reactions) = 'eps_fe56_wk_mn56'

names_weak_ps(63+ num_weak_reactions) = 'eps_co55_wk_fe55'
names_weak_ps(64+ num_weak_reactions) = 'eps_co56_wk_fe56'
names_weak_ps(65+ num_weak_reactions) = 'eps_co57_wk_fe57'
names_weak_ps(66+ num_weak_reactions) = 'eps_co58_wk_fe58'

names_weak_ps(67+ num_weak_reactions) = 'eps_ni55_wk_co55'
names_weak_ps(68+ num_weak_reactions) = 'eps_ni56_wk_co56'
names_weak_ps(69+ num_weak_reactions) = 'eps_ni57_wk_co57'
names_weak_ps(70+ num_weak_reactions) = 'eps_ni58_wk_co58'
names_weak_ps(71+ num_weak_reactions) = 'eps_ni59_wk_co59'
names_weak_ps(72+ num_weak_reactions) = 'eps_ni60_wk_co60'

names_weak_ps(73+ num_weak_reactions) = 'eps_cu59_wk_ni59'
names_weak_ps(74+ num_weak_reactions) = 'eps_cu60_wk_ni60'
names_weak_ps(75+ num_weak_reactions) = 'eps_cu61_wk_ni61'

names_weak_ps(76+ num_weak_reactions) = 'eps_zn60_wk_cu60'
names_weak_ps(77+ num_weak_reactions) = 'eps_zn61_wk_cu61'
names_weak_ps(78+ num_weak_reactions) = 'eps_zn62_wk_cu62'

! 70 inverse beta decay reactions
names_weak_ps(79+ num_weak_reactions) = 'eps_neut_wk-minus_h1'

names_weak_ps(80+ num_weak_reactions) = 'eps_o17_wk-minus_f17'
names_weak_ps(81+ num_weak_reactions) = 'eps_o18_wk-minus_f18'

names_weak_ps(82+ num_weak_reactions) = 'eps_f18_wk-minus_ne18'
names_weak_ps(83+ num_weak_reactions) = 'eps_f19_wk-minus_ne19'
names_weak_ps(84+ num_weak_reactions) = 'eps_f20_wk-minus_ne20'
names_weak_ps(85+ num_weak_reactions) = 'eps_f23_wk-minus_ne23'

names_weak_ps(86+ num_weak_reactions) = 'eps_ne21_wk-minus_na21'
names_weak_ps(87+ num_weak_reactions) = 'eps_ne22_wk-minus_na22'
names_weak_ps(88+ num_weak_reactions) = 'eps_ne23_wk-minus_na23'
names_weak_ps(89+ num_weak_reactions) = 'eps_ne24_wk-minus_na24'

names_weak_ps(90+ num_weak_reactions) = 'eps_na22_wk-minus_mg22'
names_weak_ps(91+ num_weak_reactions) = 'eps_na23_wk-minus_mg23'
names_weak_ps(92+ num_weak_reactions) = 'eps_na24_wk-minus_mg24'
names_weak_ps(93+ num_weak_reactions) = 'eps_na25_wk-minus_mg25'

names_weak_ps(94+ num_weak_reactions) = 'eps_mg25_wk-minus_al25'
names_weak_ps(95+ num_weak_reactions) = 'eps_mg26_wk-minus_al26'
names_weak_ps(96+ num_weak_reactions) = 'eps_mg27_wk-minus_al27'

names_weak_ps(97+ num_weak_reactions) = 'eps_al26_wk-minus_si26'
names_weak_ps(98+ num_weak_reactions) = 'eps_al27_wk-minus_si27'
names_weak_ps(99+ num_weak_reactions) = 'eps_al28_wk-minus_si28'

names_weak_ps(100+ num_weak_reactions) = 'eps_si30_wk-minus_p30'

names_weak_ps(101+ num_weak_reactions) = 'eps_p31_wk-minus_s31'
names_weak_ps(102+ num_weak_reactions) = 'eps_p32_wk-minus_s32'

names_weak_ps(103+ num_weak_reactions) = 'eps_cl35_wk-minus_ar35'
names_weak_ps(104+ num_weak_reactions) = 'eps_cl36_wk-minus_ar36'
names_weak_ps(105+ num_weak_reactions) = 'eps_cl37_wk-minus_ar37'

names_weak_ps(106+ num_weak_reactions) = 'eps_k39_wk-minus_ca39'
names_weak_ps(107+ num_weak_reactions) = 'eps_k40_wk-minus_ca40'
names_weak_ps(108+ num_weak_reactions) = 'eps_k41_wk-minus_ca41'
names_weak_ps(109+ num_weak_reactions) = 'eps_k42_wk-minus_ca42'
names_weak_ps(110+ num_weak_reactions) = 'eps_k43_wk-minus_ca43'

names_weak_ps(111+ num_weak_reactions) = 'eps_ca43_wk-minus_sc43'
names_weak_ps(112+ num_weak_reactions) = 'eps_ca44_wk-minus_sc44'

names_weak_ps(113+ num_weak_reactions) = 'eps_sc44_wk-minus_ti44'
names_weak_ps(114+ num_weak_reactions) = 'eps_sc45_wk-minus_ti45'
names_weak_ps(115+ num_weak_reactions) = 'eps_sc46_wk-minus_ti46'

names_weak_ps(116+ num_weak_reactions) = 'eps_ti47_wk-minus_v47'
names_weak_ps(117+ num_weak_reactions) = 'eps_ti48_wk-minus_v48'

names_weak_ps(118+ num_weak_reactions) = 'eps_v48_wk-minus_cr48'
names_weak_ps(119+ num_weak_reactions) = 'eps_v49_wk-minus_cr49'
names_weak_ps(120+ num_weak_reactions) = 'eps_v50_wk-minus_cr50'
names_weak_ps(121+ num_weak_reactions) = 'eps_v51_wk-minus_cr51'

names_weak_ps(122+ num_weak_reactions) = 'eps_cr51_wk-minus_mn51'
names_weak_ps(123+ num_weak_reactions) = 'eps_cr52_wk-minus_mn52'
names_weak_ps(124+ num_weak_reactions) = 'eps_cr53_wk-minus_mn53'
names_weak_ps(125+ num_weak_reactions) = 'eps_cr54_wk-minus_mn54'
names_weak_ps(126+ num_weak_reactions) = 'eps_cr55_wk-minus_mn55'
names_weak_ps(127+ num_weak_reactions) = 'eps_cr56_wk-minus_mn56'

names_weak_ps(128+ num_weak_reactions) = 'eps_mn52_wk-minus_fe52'
names_weak_ps(129+ num_weak_reactions) = 'eps_mn53_wk-minus_fe53'
names_weak_ps(130+ num_weak_reactions) = 'eps_mn54_wk-minus_fe54'
names_weak_ps(131+ num_weak_reactions) = 'eps_mn55_wk-minus_fe55'
names_weak_ps(132+ num_weak_reactions) = 'eps_mn56_wk-minus_fe56'

names_weak_ps(133+ num_weak_reactions) = 'eps_fe55_wk-minus_co55'
names_weak_ps(134+ num_weak_reactions) = 'eps_fe56_wk-minus_co56'
names_weak_ps(135+ num_weak_reactions) = 'eps_fe57_wk-minus_co57'
names_weak_ps(136+ num_weak_reactions) = 'eps_fe58_wk-minus_co58'

names_weak_ps(137+ num_weak_reactions) = 'eps_co55_wk-minus_ni55'
names_weak_ps(138+ num_weak_reactions) = 'eps_co56_wk-minus_ni56'
names_weak_ps(139+ num_weak_reactions) = 'eps_co57_wk-minus_ni57'
names_weak_ps(140+ num_weak_reactions) = 'eps_co58_wk-minus_ni58'
names_weak_ps(141+ num_weak_reactions) = 'eps_co59_wk-minus_ni59'
names_weak_ps(142+ num_weak_reactions) = 'eps_co60_wk-minus_ni60'

names_weak_ps(143+ num_weak_reactions) = 'eps_ni59_wk-minus_cu59'
names_weak_ps(144+ num_weak_reactions) = 'eps_ni60_wk-minus_cu60'
names_weak_ps(145+ num_weak_reactions) = 'eps_ni61_wk-minus_cu61'

names_weak_ps(146+ num_weak_reactions) = 'eps_cu60_wk-minus_zn60'
names_weak_ps(147+ num_weak_reactions) = 'eps_cu61_wk-minus_zn61'
names_weak_ps(148+ num_weak_reactions) = 'eps_cu62_wk-minus_zn62'



baryonic_mass = (/&
3d0,1d0,1d0,1d0,&
7d0,&
8d0,&
13d0,&
14d0,15d0,&
17d0,18d0,&
18d0,19d0,20d0,23d0,&
21d0,22d0,23d0,24d0,&
22d0,23d0,24d0,25d0,&
25d0,26d0,27d0,&
26d0,27d0,28d0,&
30d0,&
31d0,32d0,&
35d0,36d0,37d0,&
39d0,40d0,41d0,42d0,43d0,&
43d0,44d0,&
44d0,45d0,46d0,&
47d0,48d0,&
48d0,49d0,50d0,51d0,&
51d0,52d0,53d0,54d0,55d0,56d0,&
52d0,53d0,54d0,55d0,56d0,&
55d0,56d0,57d0,58d0,&
55d0,56d0,57d0,58d0,59d0,60d0,&
60d0,61d0,62d0,&
59d0,60d0,61d0,&

1d0,&
17d0,18d0,&
18d0,19d0,20d0,23d0,&
21d0,22d0,23d0,24d0,&
22d0,23d0,24d0,25d0,&
25d0,26d0,27d0,&
26d0,27d0,28d0,&
30d0,&
31d0,32d0,&
35d0,36d0,37d0,&
39d0,40d0,41d0,42d0,43d0,&
43d0,44d0,&
44d0,45d0,46d0,&
47d0,48d0,&
48d0,49d0,50d0,51d0,&
52d0,52d0,53d0,54d0,55d0,56d0,&
52d0,53d0,54d0,55d0,56d0,&
55d0,56d0,57d0,58d0,&
55d0,56d0,57d0,58d0,59d0,60d0,&
59d0,60d0,61d0,&
60d0,61d0,62d0/)


do i=1,296
    names(i) = names_weak_ps(i)
end do


names(297)='log_Lneu_plas' ! units of Lsun
names(298)='log_Lneu_brem' ! units of Lsun
names(299)='log_Lneu_phot' ! units of Lsun
names(300)='log_Lneu_pair' ! units of Lsun
names(301)='log_Lneu_reco' ! units of Lsun
names(302)='gamma1_integral' ! integrated <gamma1-4/3>
names(303) = 'log_Lneu_beta' ! units of Lsun
names(304) = 'log_Lneu_invbeta' ! units of Lsun
names(305) = 'log_nps_beta' ! units of 1/s
names(306) = 'log_nps_invbeta' ! units of 1/s






!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! We begin the calculation here:
! set initial values to 0
integral_norm = 0d0
gamma1_integral = 0d0
do i=1,306
    vals(i) = 0
end do

!!!!!! here, do you want to calculate rates? okay = .true.
! calculare values or not. yes or no.
okay = .true.
! $OMP PARALLEL DO PRIVATE(k,op_err)
net_lwork = net_work_size(s% net_handle, ierr)


do k = 1, s% nz
    !if (.not. okay) cycle
    op_err = 0
    ! set initial values to 0
    do i=1,306
    temp(i) = vals(i)
    end do


    ! now do rates
    ! when this function gets called it sets all the vals(n)
    !to new values associated with the screening mode in current cell k
    call do2_net( &
    s, k, s% species, &
    s% num_reactions, net_lwork, n, s% nz, vals, frac,names_weak_reactions, ierr)
    if (op_err /= 0) okay = .false.





    !neutrinos/second for each reaction
    ! write(*,*) 'test h1 mass fraction', h1_frac
    ! Here vals have been called, so before each loop we can modify vals
    ! multiply the appropriate quantity for each rate by vals(#)
    ! s% xa(g% net_iso(reaction_inputs(2*f,rate_id)),k) example f is factor
    do i =1,num_weak_reactions ! for all weak reactions in mesa_136.net except first 3.
        if (i == 1) then
        vals(1) = exp10(vals(1)) * (frac(2) * frac(1) * s% dm(k) * avo /(baryonic_mass(2) + baryonic_mass(1))) ! neutrinos/s
     !   write (*,*) 'vals(1)', vals(1)
      ! write (*,*) '1vals(149)', vals(149)
        vals(149) = vals(149) * vals(1) /avo ! ergs/s
        vals(149) = temp(149) + vals(149)
        vals(1) = temp(1) + vals(1)

        else if (i ==2) then
        vals(2) = exp10(vals(2)) * (frac(2)**2 * s% dm(k) * avo /2d0 /baryonic_mass(2)) ! 1/s
      !  write (*,*) 'vals(2)', vals(2)
       ! write (*,*) '1vals(150)', vals(150)
        vals(150) = vals(150) * vals(2) /avo ! ergs/s, vals(28) = Qneutrino * conversion_factor_to_mev_qconv(Mev_to_ergs*avo)
        vals(150) = temp(150) + vals(150)
        vals(2) = temp(2) + vals(2)


        else if (i ==3) then
        vals(3) = exp10(vals(3)) * (frac(2)**2 * s% dm(k) * avo /2d0 /baryonic_mass(2)) ! 1/s
        !write (*,*) 'vals(3)', vals(3)
       ! write (*,*) '1vals(151)', vals(151)
        vals(151) = vals(151) * vals(3) /avo ! ergs/s, vals(28) = Qneutrino * conversion_factor_to_mev_qconv(Mev_to_ergs*avo)
        vals(151) = temp(151) + vals(151)
        vals(3) = temp(3) + vals(3)


        else
            vals(i) = exp10(vals(i)) /baryonic_mass(i) * frac(i) * s% dm(k) * avo ! neutrinos/s
    
            vals(i+num_weak_reactions) = vals(i+num_weak_reactions) * vals(i) /avo ! ergs/s
    !        write (*,*) '2vals(i+num_weak_reactions)', vals(i+num_weak_reactions)
            vals(i+num_weak_reactions) = temp(i+num_weak_reactions) + vals(i+num_weak_reactions)
     !       write (*,*) '3vals(i+num_weak_reactions)', vals(i+num_weak_reactions)
            vals(i) = temp(i) + vals(i)

        end if
    end do
    ! calulcating log L and gamma1
    ! set ending variables here:
    vals(297) = temp(297) + s% nonnucneu_plas(k) * s% dm(k)
    vals(298) = temp(298) + s% nonnucneu_brem(k) * s% dm(k)
    vals(299) = temp(299) + s% nonnucneu_phot(k) * s% dm(k)
    vals(300) = temp(300) + s% nonnucneu_pair(k) * s% dm(k)
    vals(301) = temp(301) + s% nonnucneu_reco(k) * s% dm(k)

    ! compute gamma integral in what will remain of the star
    ! temp302 unsued for gamma1
    P_tot = s% Prad(k) + s% Pgas(k) ! s% P flag is broken for some reason, so we calculate manually
    integral_norm = integral_norm + P_tot*s% dm(k)/s% rho(k)
    gamma1_integral = gamma1_integral + &
    (s% gamma1(k)-4.d0/3.d0)*P_tot*s% dm(k)/s% rho(k)

end do


vals(297) = safe_log10(vals(297)/Lsun)
vals(298) = safe_log10(vals(298)/Lsun)
vals(299) = safe_log10(vals(299)/Lsun)
vals(300) = safe_log10(vals(300)/Lsun)
vals(301) = safe_log10(vals(301)/Lsun)
gamma1_integral = gamma1_integral/max(1d-99,integral_norm)
vals(302) = gamma1_integral







eps_beta = 0
nps_beta = 0
eps_invbeta = 0
nps_invbeta = 0

do i = 1,num_weak_reactions
    if (i <= 78) then
        nps_beta =  vals(i)
        eps_beta =  vals(i+num_weak_reactions)
    else
        nps_invbeta =  vals(i)
        eps_invbeta =  vals(i+num_weak_reactions)
    end if
 !   write (*,*) 'i = ', i
    vals(303) = vals(303) + eps_beta
    vals(304) = vals(304) + eps_invbeta
    vals(305) = vals(305) + nps_beta
    vals(306) = vals(306) + nps_invbeta
!write (*,*) 'eps_beta', eps_beta
!write (*,*) 'total so far',vals(303)

end do


vals(303) = safe_log10(vals(303)/Lsun)! units of Lsun
vals(304) = safe_log10(vals(304)/Lsun) ! units of Lsun
vals(305) = safe_log10(vals(305)) ! units of 1/s
vals(306) = safe_log10(vals(306)) ! units of 1/s

end subroutine data_for_extra_history_columns

      
      integer function how_many_extra_profile_columns(id)
         use star_def, only: star_info
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_profile_columns = 1!2
      end function how_many_extra_profile_columns
      
      
      subroutine data_for_extra_profile_columns(id, n, nz, names, vals, ierr)
         use star_def, only: star_info, maxlen_profile_column_name
         use const_def, only: dp
         integer, intent(in) :: id, n, nz
         character (len=maxlen_profile_column_name) :: names(n)
         real(dp) :: vals(nz,n)
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         integer :: k
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
        names(1) = 'zbar_div_abar'
        !names(2) = 'end_eps_neu_nuc'
        do k=1,s% nz
           vals(k,1) = s% zbar(k)/s% abar(k)
           !vals(k,2) = s% xtra1_array(k)
         !  if (vals(k,2)> 0d0):
        end do
      end subroutine data_for_extra_profile_columns

  
      include 'gyre_in_mesa_extras_finish_step.inc'
     

      ! returns either keep_going or terminate.
      integer function extras_finish_step(id)
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_finish_step = keep_going
         if (.not. s% x_logical_ctrl(37)) return
         extras_finish_step = gyre_in_mesa_extras_finish_step(id)
         if (extras_finish_step == terminate) &
             s% termination_code = t_extras_finish_step
      end function extras_finish_step
      
! std_reaction_neuQs is from reaclib or weak_info.list

    subroutine do2_net( &
    s, k, species, num_reactions, net_lwork, &
    n, nz, vals, frac, names_weak_reactions, ierr)
    use rates_def, only: std_reaction_Qs, std_reaction_neuQs, i_rate, reaction_data
    use net_def, only: Net_Info
    use net_lib, only: net_get_with_Qs
    use chem_def, only: chem_isos, category_name
    use eos_def, only : i_eta
    use utils_lib,only: &
    is_bad_num, realloc_double, realloc_double3
    type (star_info), pointer :: s
    integer, intent(in) :: k, species, num_reactions, net_lwork, n, nz
    character (len=maxlen_history_column_name) :: names_weak_reactions(n)
    real(dp) :: vals(nz,n)
    real(dp) :: frac(n)
    integer, intent(out) :: ierr

    integer :: i, j, screening_mode
    real(dp) :: log10_rho, log10_T, alfa, beta, &
    d_eps_nuc_dRho, d_eps_nuc_dT, cat_factor
    real(dp), target :: net_work_ary(net_lwork)
    real(dp), pointer :: net_work(:)
    type (Net_Info), target :: net_info_target
    type (Net_Info), pointer :: netinfo

    character (len=100) :: message
    !real(dp), pointer :: reaction_neuQs(:)
    real(dp) :: eps_neu_tot ! ergs/g/s neutrinos from weak reactions
    real(dp), pointer, dimension(:) :: actual_Qs_p, actual_neuQs_p ! ignore if null  (num_reactions)
    logical, pointer :: from_weaklib_p(:) ! ignore if null
    integer :: sz
    real(dp) :: eps_nuc_factor

    logical, parameter :: dbg = .false.

    include 'formats'


    ierr = 0

    net_work => net_work_ary
    netinfo => net_info_target


     actual_neuQs_p => actual_neuQs
     actual_Qs_p => actual_Qs
     from_weaklib_p => from_weaklib

    log10_rho = s% lnd(k)/ln10
    log10_T = s% lnT(k)/ln10

    screening_mode = get_screening_mode(s,ierr)
    if (ierr /= 0) then
    write(*,*) 'unknown string for screening_mode: ' // trim(s% screening_mode)
    stop 'do2_net'
    return
    end if

    call net_get_with_Qs( &
    s% net_handle, .false., netinfo, species, num_reactions, s% xa(1:species,k), &
    s% T(k), log10_T, s% rho(k), log10_Rho, &
    s% abar(k), s% zbar(k), s% z2bar(k), s% ye(k), &
    s% eta(k), s% d_eos_dlnT(i_eta,k) , s% d_eos_dlnd(i_eta,k), &
    s% rate_factors, s% weak_rate_factor, &
    std_reaction_Qs, std_reaction_neuQs, .false., .false., &
    s% eps_nuc(k), d_eps_nuc_dRho, d_eps_nuc_dT, s% d_epsnuc_dx(:,k), &
    s% dxdt_nuc(:,k), s% d_dxdt_nuc_dRho(:,k), s% d_dxdt_nuc_dT(:,k), s% d_dxdt_nuc_dx(:,:,k), &
    screening_mode, s% theta_e(k), &
    s% eps_nuc_categories(:,k), &
    eps_neu_tot, net_lwork,net_work,actual_Qs_p, actual_neuQs_p, from_weaklib_p, ierr)
    

    if (ierr /= 0) then
    write(*,*) 'do2_net: net_get failure for cell ', k
    return
    end if
    call show_stuff1(s,k,net_lwork,net_work,n,nz,vals,frac,names_weak_reactions)

    end subroutine do2_net




    integer function get_screening_mode(s,ierr)
    use rates_lib, only: screening_option
    type (star_info), pointer :: s
    integer, intent(out) :: ierr
    include 'formats'
    ierr = 0
    if (s% screening_mode_value >= 0) then
    get_screening_mode = s% screening_mode_value
    return
    end if
    get_screening_mode = screening_option(s% screening_mode, ierr)
    if (ierr /= 0) return
    s% screening_mode_value = get_screening_mode
    ! returns a value sets which screening option to choose.
    !write(*,2) 'get_screening_mode ' // &
    !   trim(s% screening_mode), get_screening_mode
    end function get_screening_mode




    subroutine show_stuff1(s,k,lwork,work,n,nz,vals,frac,names_weak_reactions)
    use chem_def
    use rates_def
    use rates_lib, only: rates_reaction_id
    use net_lib, only: get_reaction_id_table_ptr, get_net_rate_ptrs
    use net_def, only: Net_Info, Net_General_Info, get_net_ptr
    use math_lib, only: safe_log10
    type (star_info), pointer :: s
    integer, intent(in) :: k, lwork, n, nz!, num_weak_reactions
    character (len=maxlen_history_column_name) :: names_weak_reactions(n)
    real(dp), pointer :: work(:)
    real(dp) :: vals(n), frac(n)
    integer :: rate_id(n)
    type (Net_General_Info), pointer :: g


    integer, pointer :: reaction_id(:) ! maps net reaction number to reaction id
    integer :: i, j, ierr, species, num_reactions!, rate_id
    real(dp), pointer, dimension(:) :: &
    rate_screened, rate_screened_dT, rate_screened_dRho, &
    rate_raw, rate_raw_dT, rate_raw_dRho
    !real(dp), pointer, dimension(:), intent(in) :: actual_Qs, actual_neuQs ! ignore if null  (num_reactions)
    !logical, pointer, intent(in) :: from_weaklib(:) ! ignore if null

    include 'formats'

    ierr = 0
    num_reactions = s% num_reactions


    call get_net_ptr(s% net_handle, g, ierr)

    call get_net_rate_ptrs(s% net_handle, &
    rate_screened, rate_screened_dT, rate_screened_dRho, &
    rate_raw, rate_raw_dT, rate_raw_dRho, lwork, work, &
    ierr)
    if (ierr /= 0) then
    write(*,*) 'failed in get_net_rate_ptrs'
    stop 1
    end if

    call get_reaction_id_table_ptr(s% net_handle, reaction_id, ierr)
    if (ierr /= 0) return

!weaklib_ids
! We calculate the first 3 rates explicitly.
    !r_h1_he3_wk_he4

    rate_id(1) = rates_reaction_id(names_weak_reactions(1))
    if (rate_id(1) <= 0) then
    write(*,*) 'failed to find reaction rate id 1 -- not valid name?'
    vals(1) = 0
    return
    end if
    call get_rate(rate_id(1),vals(1),vals(1+148),k)
    frac(1) = s% xa(g% net_iso(reaction_inputs(4,rate_id(1))),k)


    !r_h1_h1_ec_h2
    rate_id(2) = rates_reaction_id(names_weak_reactions(2))
    if (rate_id(2) <= 0) then
    write(*,*) 'failed to find reaction rate id 2 -- not valid name?'
    vals(2) = 0
    return
    end if
    call get_rate(rate_id(2),vals(2),vals(2+148),k)
    frac(2) = s% xa(g% net_iso(reaction_inputs(1,rate_id(2))),k)
    !write(*,*) 'h1 ', frac(2) ! check h1 frac

    !r_h1_h1_wk_h2
    rate_id(3) = rates_reaction_id(names_weak_reactions(3))
    if (rate_id(3) <= 0) then
    write(*,*) 'failed to find reaction rate id 3 -- not valid name?'
    vals(3) = 0
    return
    end if
    call get_rate(rate_id(3),vals(3),vals(3+148),k)
    frac(3) = s% xa(g% net_iso(reaction_inputs(1,rate_id(3))),k)

    ! all other reactions
    do i = 4,148
        rate_id(i) = rates_reaction_id(names_weak_reactions(i))
        if (rate_id(i) <= 0) then
        write(*,*) 'failed to find reaction rate id ', i ,' -- not valid name?'
        vals(i) = 0
        return
        end if
        call get_rate(rate_id(i),vals(i),vals(i+148),k)
        frac(i) = s% xa(g% net_iso(reaction_inputs(2,rate_id(i))),k)
       ! write(*,*)' i = ', i
        !write(*,*) 'vals(i+148), neuqs', vals(i+148)
    end do

    !write (*,*) 'k', k
    !write(*,*) 'he3', he3_frac
    !write(*,*) 'h1', h1_frac
    !write(*,*) 'be7', be7_frac
    !write(*,*) 'ne19', ne19_frac
    !write(*,*) 'si27', si27_frac



    contains

    subroutine get_rate(id, get_rate1, get_rate_Qneu,k1)
    integer, intent(in) :: id
    !real(dp), intent(in) :: get_rate1, get_rate_Qneu
    integer :: j, k1
    real(dp), intent(out):: get_rate1, get_rate_Qneu

    include 'formats'

    get_rate1 = -99
    get_rate_Qneu = 0
    do j=1,num_reactions
    if (reaction_id(j) /= id) cycle
    !write(*,3) 'screened rate ' // trim(reaction_Name(reaction_id(j))), &
    !   j, k, rate_screened(j)
    if (rate_screened(j) < 1d-60) then
    get_rate1 = -99d0
    get_rate_Qneu = 0
    else
    get_rate1 = safe_log10(rate_screened(j)) ! or rate_raw(j)
    get_rate_Qneu = Qconv * actual_neuQs(j) !std_reaction_neuQs(id)

    if (k1 == s% nz) then
    !write (*,*) s% weak_rate_factor !std_reaction_neuQs(id)
    end if
    end if
    return
    end do

    write(*,*) 'failed to find reaction rate id -- not in current net?'
    get_rate1 = -99
    get_rate_Qneu = 0
    end subroutine get_rate
    end subroutine show_stuff1

      end module run_star_extras
      
